package net.t2code.opsecurity.events;

import net.t2code.opsecurity.config.language.Languages;
import net.t2code.opsecurity.config.opWhitelist.OPWhitelist;
import net.t2code.opsecurity.enums.OpCommandRequest;
import net.t2code.opsecurity.objects.PlayerCache;
import net.t2code.opsecurity.objects.PlayerObject;
import net.t2code.opsecurity.system.Main;
import net.t2code.t2codelib.SPIGOT.api.debug.T2C_Debug;
import net.t2code.t2codelib.SPIGOT.api.messages.T2C_Send;
import net.t2code.t2codelib.SPIGOT.api.player.T2C_NameHistory;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerCommandPreprocessEvent;
import org.bukkit.event.server.ServerCommandEvent;

import java.io.IOException;
import java.util.Map;

public class OpCommand implements Listener {
    @EventHandler
    public void onOPServer(ServerCommandEvent event) {
        if (!(boolean) OPWhitelist.VALUES.enable.getValue()) return;
        if ((event.getCommand().toLowerCase().startsWith("op ") || event.getCommand().toLowerCase().startsWith("minecraft:op "))) {
            switch (isNotOPWTL(event.getCommand())) {
                case mustOnline:
                    T2C_Send.sender(event.getSender(), Languages.VALUES.playerMustBeOnlineToOp.getValue().toString());
                    event.setCancelled(true);
                    break;
                case notWhitelisted:
                    T2C_Send.sender(event.getSender(), Languages.VALUES.opWhitelistOpCommand.getValue());
                    event.setCancelled(true);
                    break;
            }
        }
    }

    @EventHandler
    public void onOpPlayer(PlayerCommandPreprocessEvent event) {
        if (!(boolean) OPWhitelist.VALUES.enable.getValue()) return;
        if ((event.getMessage().toLowerCase().startsWith("/op ") || event.getMessage().toLowerCase().startsWith("/minecraft:op "))) {
            switch (isNotOPWTL(event.getMessage())) {
                case mustOnline:
                    T2C_Send.player(event.getPlayer(), Languages.VALUES.playerMustBeOnlineToOp.getValue().toString());
                    event.setCancelled(true);
                    break;
                case notWhitelisted:
                    T2C_Send.player(event.getPlayer(), Languages.VALUES.opWhitelistOpCommand.getValue().toString());
                    event.setCancelled(true);
                    break;
            }
        }
    }

    private OpCommandRequest isNotOPWTL(String command) {
        if (command.charAt(0) == '/') command = command.replaceFirst("/", "");
        String arg = command.replace("op ", "");
        Player target = Bukkit.getPlayer(arg);

        if ((boolean)OPWhitelist.VALUES.playerMustBeOnlineToOp.getValue()) {
            if (target == null) return OpCommandRequest.mustOnline;
            if (!opWhitelist(target.getName(), target.getUniqueId().toString())) return OpCommandRequest.notWhitelisted;
        }

        String targetUUID;
        if (target != null) {
            targetUUID = target.getUniqueId().toString();
        } else {
            try {
                targetUUID = T2C_NameHistory.NameLookup.getPlayerUUID(arg);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
        if (!opWhitelist(arg, targetUUID)) return OpCommandRequest.notWhitelisted;
        return OpCommandRequest.ok;
    }

    private static Boolean opWhitelist(String playerName, String playerUuid) {
        for (Map.Entry<String, PlayerObject> playerObject : PlayerCache.getOpHashMap().entrySet()) {
            T2C_Debug.debug(Main.getPlugin(), "--------- " + playerName);
            T2C_Debug.debug(Main.getPlugin(),"opWhitelist (op command) playerObject Name: "+playerObject.getValue().playerName );
            T2C_Debug.debug(Main.getPlugin(),"opWhitelist (op command) Player Name: "+ playerName );
            T2C_Debug.debug(Main.getPlugin(),"opWhitelist (op command) playerObject UUID: "+playerObject.getValue().uuid );
            T2C_Debug.debug(Main.getPlugin(),"opWhitelist (op command) Player UUID: "+ playerUuid );
            if (playerObject.getValue().playerName.equals(playerName) && playerObject.getValue().uuid.equals(playerUuid.replace("-", ""))) {
                T2C_Debug.debug(Main.getPlugin(),"opWhitelist (op command) on list!");
                return true;
            }
        }
        return false;
    }
}
